import requests


from .Bridge import Bridge
from .BridgeCreate import BridgeCreate
from .BridgeCreateSetting import BridgeCreateSetting
from .CPUInfo import CPUInfo
from .CPUStats import CPUStats
from .CloudInit import CloudInit
from .Cluster import Cluster
from .ClusterCreate import ClusterCreate
from .Container import Container
from .ContainerListItem import ContainerListItem
from .ContainerNIC import ContainerNIC
from .ContainerNICconfig import ContainerNICconfig
from .CoreStateResult import CoreStateResult
from .CoreSystem import CoreSystem
from .CreateContainer import CreateContainer
from .CreateSnapshotReqBody import CreateSnapshotReqBody
from .DHCP import DHCP
from .DeleteFile import DeleteFile
from .DiskInfo import DiskInfo
from .DiskPartition import DiskPartition
from .EnumBridgeCreateNetworkMode import EnumBridgeCreateNetworkMode
from .EnumBridgeStatus import EnumBridgeStatus
from .EnumClusterCreateClusterType import EnumClusterCreateClusterType
from .EnumClusterCreateDriveType import EnumClusterCreateDriveType
from .EnumClusterDriveType import EnumClusterDriveType
from .EnumClusterStatus import EnumClusterStatus
from .EnumContainerListItemStatus import EnumContainerListItemStatus
from .EnumContainerNICStatus import EnumContainerNICStatus
from .EnumContainerNICType import EnumContainerNICType
from .EnumContainerStatus import EnumContainerStatus
from .EnumDiskInfoType import EnumDiskInfoType
from .EnumGWNICType import EnumGWNICType
from .EnumJobResultName import EnumJobResultName
from .EnumJobResultState import EnumJobResultState
from .EnumNicLinkType import EnumNicLinkType
from .EnumNodeStatus import EnumNodeStatus
from .EnumStoragePoolCreateDataProfile import EnumStoragePoolCreateDataProfile
from .EnumStoragePoolCreateMetadataProfile import EnumStoragePoolCreateMetadataProfile
from .EnumStoragePoolDataProfile import EnumStoragePoolDataProfile
from .EnumStoragePoolDeviceStatus import EnumStoragePoolDeviceStatus
from .EnumStoragePoolListItemStatus import EnumStoragePoolListItemStatus
from .EnumStoragePoolMetadataProfile import EnumStoragePoolMetadataProfile
from .EnumStoragePoolStatus import EnumStoragePoolStatus
from .EnumStorageServerStatus import EnumStorageServerStatus
from .EnumVMListItemStatus import EnumVMListItemStatus
from .EnumVMStatus import EnumVMStatus
from .EnumVdiskCreateType import EnumVdiskCreateType
from .EnumVdiskListItemStatus import EnumVdiskListItemStatus
from .EnumVdiskListItemType import EnumVdiskListItemType
from .EnumVdiskStatus import EnumVdiskStatus
from .EnumVdiskType import EnumVdiskType
from .EnumZerotierListItemType import EnumZerotierListItemType
from .EnumZerotierType import EnumZerotierType
from .Filesystem import Filesystem
from .FilesystemCreate import FilesystemCreate
from .GW import GW
from .GWCreate import GWCreate
from .GWHost import GWHost
from .GWNIC import GWNIC
from .GWNICconfig import GWNICconfig
from .GetGW import GetGW
from .HTTPProxy import HTTPProxy
from .HTTPType import HTTPType
from .IPProtocol import IPProtocol
from .Job import Job
from .JobListItem import JobListItem
from .JobResult import JobResult
from .ListGW import ListGW
from .MemInfo import MemInfo
from .NicInfo import NicInfo
from .NicLink import NicLink
from .Node import Node
from .NodeMount import NodeMount
from .OSInfo import OSInfo
from .PortForward import PortForward
from .Process import Process
from .ProcessSignal import ProcessSignal
from .Run import Run
from .RunState import RunState
from .Snapshot import Snapshot
from .StoragePool import StoragePool
from .StoragePoolCreate import StoragePoolCreate
from .StoragePoolDevice import StoragePoolDevice
from .StoragePoolListItem import StoragePoolListItem
from .StorageServer import StorageServer
from .VDiskLink import VDiskLink
from .VM import VM
from .VMCreate import VMCreate
from .VMDiskInfo import VMDiskInfo
from .VMInfo import VMInfo
from .VMListItem import VMListItem
from .VMMigrate import VMMigrate
from .VMNicInfo import VMNicInfo
from .VMUpdate import VMUpdate
from .Vdisk import Vdisk
from .VdiskCreate import VdiskCreate
from .VdiskListItem import VdiskListItem
from .VdiskResize import VdiskResize
from .VdiskRollback import VdiskRollback
from .WriteFile import WriteFile
from .Zerotier import Zerotier
from .ZerotierBridge import ZerotierBridge
from .ZerotierJoin import ZerotierJoin
from .ZerotierListItem import ZerotierListItem
from .ZerotierRoute import ZerotierRoute

from .client import Client as APIClient

from .oauth2_client_itsyouonline import Oauth2ClientItsyouonline

class Client:
    def __init__(self, base_uri=""):
        self.api = APIClient(base_uri)
        
        self.oauth2_client_itsyouonline = Oauth2ClientItsyouonline()


from JumpScale.sal.g8os.Node import Node
from JumpScale.sal.g8os.StorageCluster import StorageCluster
from js9 import j

class G8OSFactory(object):
    """Factory for G8OS SAL"""

    def __init__(self):
        self.__jslocation__ = "j.sal.g8os"
        self.logger = j.logger.get('j.sal.g8os')

    def get_node(self, addr, port=6379, password=None):
        """
        Returns a Node object that represent a G8OS node reachable
        at addr:port
        """
        return Node(
            addr=addr,
            port=port,
            password=password,
        )

    def create_storagecluster(self, label, nodes, disk_type, nr_server, has_slave=True):
        """
        @param label: string repsenting the name of the storage cluster
        @param nodes: list of node on wich we can deploy storage server
        @param disk_type: type of disk to be used by the storage server
        @param nr_server: number of storage server to deploy
        @param has_slave: boolean specifying of we need to deploy slave storage server
        """
        cluster = StorageCluster(label=label)
        cluster.create(nodes=nodes, disk_type=disk_type, nr_server=nr_server, has_slave=has_slave)
        return cluster


if __name__ == '__main__':
    from js9 import j
    node1 = j.sal.g8os.get_node('172.20.0.91')
    node2 = j.sal.g8os.get_node('172.20.0.92')
    cluster = j.sal.g8os.create_storagecluster('cluster1',[node1, node2],'hdd', 2, False)

    from IPython import embed;embed()



"""
Auto-generated class for Zerotier
"""
from .EnumZerotierType import EnumZerotierType
from .ZerotierRoute import ZerotierRoute

from . import client_support


class Zerotier(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(allowDefault, allowGlobal, allowManaged, assignedAddresses, bridge, broadcastEnabled, dhcp, mac, mtu, name, netconfRevision, nwid, portDeviceName, portError, routes, status, type):
        """
        :type allowDefault: bool
        :type allowGlobal: bool
        :type allowManaged: bool
        :type assignedAddresses: list[str]
        :type bridge: bool
        :type broadcastEnabled: bool
        :type dhcp: bool
        :type mac: str
        :type mtu: int
        :type name: str
        :type netconfRevision: int
        :type nwid: str
        :type portDeviceName: str
        :type portError: int
        :type routes: list[ZerotierRoute]
        :type status: str
        :type type: EnumZerotierType
        :rtype: Zerotier
        """

        return Zerotier(
            allowDefault=allowDefault,
            allowGlobal=allowGlobal,
            allowManaged=allowManaged,
            assignedAddresses=assignedAddresses,
            bridge=bridge,
            broadcastEnabled=broadcastEnabled,
            dhcp=dhcp,
            mac=mac,
            mtu=mtu,
            name=name,
            netconfRevision=netconfRevision,
            nwid=nwid,
            portDeviceName=portDeviceName,
            portError=portError,
            routes=routes,
            status=status,
            type=type,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'Zerotier'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'allowDefault'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.allowDefault = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'allowGlobal'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.allowGlobal = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'allowManaged'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.allowManaged = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'assignedAddresses'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.assignedAddresses = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'bridge'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.bridge = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'broadcastEnabled'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.broadcastEnabled = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'dhcp'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.dhcp = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'mac'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.mac = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'mtu'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.mtu = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'name'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.name = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'netconfRevision'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.netconfRevision = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'nwid'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.nwid = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'portDeviceName'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.portDeviceName = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'portError'
        val = data.get(property_name)
        if val is not None:
            datatypes = [int]
            try:
                self.portError = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'routes'
        val = data.get(property_name)
        if val is not None:
            datatypes = [ZerotierRoute]
            try:
                self.routes = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'status'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.status = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'type'
        val = data.get(property_name)
        if val is not None:
            datatypes = [EnumZerotierType]
            try:
                self.type = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



from zeroos.orchestrator.sal import templates
import signal
import time


class HTTPServer:
    def __init__(self, container, httpproxies):
        self.container = container
        self.httpproxies = httpproxies

    def apply_rules(self):
        # caddy
        caddyconfig = templates.render('caddy.conf', httpproxies=self.httpproxies)
        self.container.upload_content('/etc/caddy.conf', caddyconfig)
        job = self.get_job()
        if job:
            self.container.client.job.kill(job['cmd']['id'], int(signal.SIGUSR1))
        else:
            self.container.client.system('caddy -agree -conf /etc/caddy.conf', stdin='\n')
        start = time.time()
        while start + 10 > time.time():
            if self.is_running():
                return True
            time.sleep(0.5)
        raise RuntimeError("Failed to start caddy server")

    def get_job(self):
        for job in self.container.client.job.list():
            cmd = job['cmd']
            if cmd['command'] != 'core.system':
                continue
            if cmd['arguments']['name'] == 'caddy':
                return job

    def is_running(self):
        for port in self.container.client.info.port():
            if port['network'].startswith('tcp') and port['port'] in [80, 443]:
                return True
        return False



import signal
import time

from zeroos.orchestrator.sal import templates

DNSMASQ = '/bin/dnsmasq --conf-file=/etc/dnsmasq.conf -d'


class DHCP:
    def __init__(self, container, domain, dhcps):
        self.container = container
        self.domain = domain
        self.dhcps = dhcps

    def apply_config(self):
        dnsmasq = templates.render('dnsmasq.conf', domain=self.domain, dhcps=self.dhcps)
        self.container.upload_content('/etc/dnsmasq.conf', dnsmasq)

        dhcp = templates.render('dhcp', dhcps=self.dhcps)
        self.container.upload_content('/etc/dhcp', dhcp)

        for process in self.container.client.process.list():
            if 'dnsmasq' in process['cmdline']:
                self.container.client.process.kill(process['pid'], signal.SIGTERM)
                start = time.time()
                while start + 10 > time.time():
                    if not self.is_running():
                        break
                    time.sleep(0.2)
                break

        self.container.client.system(DNSMASQ)
        # check if command is listening for dhcp
        start = time.time()
        while start + 10 > time.time():
            if self.is_running():
                break
            time.sleep(0.2)
        else:
            raise RuntimeError('Failed to run dnsmasq')

    def is_running(self):
        for port in self.container.client.info.port():
            if port['network'] == 'udp' and port['port'] == 53:
                return True



"""
Auto-generated class for ContainerNIC
"""
from .ContainerNICconfig import ContainerNICconfig
from .EnumContainerNICStatus import EnumContainerNICStatus
from .EnumContainerNICType import EnumContainerNICType

from . import client_support


class ContainerNIC(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(id, status, type, config=None, hwaddr=None, name=None, token=None):
        """
        :type config: ContainerNICconfig
        :type hwaddr: str
        :type id: str
        :type name: str
        :type status: EnumContainerNICStatus
        :type token: str
        :type type: EnumContainerNICType
        :rtype: ContainerNIC
        """

        return ContainerNIC(
            config=config,
            hwaddr=hwaddr,
            id=id,
            name=name,
            status=status,
            token=token,
            type=type,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'ContainerNIC'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'config'
        val = data.get(property_name)
        if val is not None:
            datatypes = [ContainerNICconfig]
            try:
                self.config = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'hwaddr'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.hwaddr = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'id'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.id = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'name'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.name = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'status'
        val = data.get(property_name)
        if val is not None:
            datatypes = [EnumContainerNICStatus]
            try:
                self.status = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'token'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.token = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'type'
        val = data.get(property_name)
        if val is not None:
            datatypes = [EnumContainerNICType]
            try:
                self.type = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



"""
Auto-generated class for Container
"""
from .ContainerNIC import ContainerNIC
from .CoreSystem import CoreSystem
from .EnumContainerStatus import EnumContainerStatus

from . import client_support


class Container(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(filesystems, flist, hostNetworking, hostname, initprocesses, nics, ports, status, storage, zerotiernodeid=None):
        """
        :type filesystems: list[str]
        :type flist: str
        :type hostNetworking: bool
        :type hostname: str
        :type initprocesses: list[CoreSystem]
        :type nics: list[ContainerNIC]
        :type ports: list[str]
        :type status: EnumContainerStatus
        :type storage: str
        :type zerotiernodeid: str
        :rtype: Container
        """

        return Container(
            filesystems=filesystems,
            flist=flist,
            hostNetworking=hostNetworking,
            hostname=hostname,
            initprocesses=initprocesses,
            nics=nics,
            ports=ports,
            status=status,
            storage=storage,
            zerotiernodeid=zerotiernodeid,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'Container'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'filesystems'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.filesystems = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'flist'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.flist = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'hostNetworking'
        val = data.get(property_name)
        if val is not None:
            datatypes = [bool]
            try:
                self.hostNetworking = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'hostname'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.hostname = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'initprocesses'
        val = data.get(property_name)
        if val is not None:
            datatypes = [CoreSystem]
            try:
                self.initprocesses = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'nics'
        val = data.get(property_name)
        if val is not None:
            datatypes = [ContainerNIC]
            try:
                self.nics = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'ports'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.ports = client_support.list_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'status'
        val = data.get(property_name)
        if val is not None:
            datatypes = [EnumContainerStatus]
            try:
                self.status = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'storage'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.storage = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'zerotiernodeid'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.zerotiernodeid = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)



import json

from js9 import j
from .StorageEngine import StorageEngine

import logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class StorageCluster:
    """StorageCluster is a cluster of StorageEngine servers"""

    def __init__(self, label, nodes=None, disk_type=None):
        """
        @param label: string repsenting the name of the storage cluster
        """
        self.label = label
        self.name = label
        self.nodes = nodes or []
        self.filesystems = []
        self.storage_servers = []
        self.disk_type = disk_type
        self.k = 0
        self.m = 0
        self._ays = None

    @classmethod
    def from_ays(cls, service, password):
        logger.debug("load cluster storage cluster from service (%s)", service)
        disk_type = str(service.model.data.diskType)

        nodes = []
        storage_servers = []
        for storageEngine_service in service.producers.get('storage_engine', []):
            storages_server = StorageServer.from_ays(storageEngine_service, password)
            storage_servers.append(storages_server)
            if storages_server.node not in nodes:
                nodes.append(storages_server.node)

        cluster = cls(label=service.name, nodes=nodes, disk_type=disk_type)
        cluster.storage_servers = storage_servers
        cluster.k = service.model.data.k
        cluster.m = service.model.data.m
        return cluster

    @property
    def dashboard(self):
        board = StorageDashboard(self)
        return board.template

    def get_config(self):
        data = {'dataStorage': [],
                'metadataStorage': None,
                'label': self.name,
                'status': 'ready' if self.is_running() else 'error',
                'nodes': [node.name for node in self.nodes]}
        for storageserver in self.storage_servers:
            if 'metadata' in storageserver.name:
                data['metadataStorage'] = {'address': storageserver.storageEngine.bind}
            else:
                data['dataStorage'].append({'address': storageserver.storageEngine.bind})
        return data

    @property
    def nr_server(self):
        """
        Number of storage server part of this cluster
        """
        return len(self.storage_servers)

    def find_disks(self):
        """
        return a list of disk that are not used by storage pool
        or has a different type as the one required for this cluster
        """
        logger.debug("find available_disks")
        cluster_name = 'sp_cluster_{}'.format(self.label)
        available_disks = {}

        def check_partition(disk):
            for partition in disk.partitions:
                for filesystem in partition.filesystems:
                    if filesystem['label'].startswith(cluster_name):
                        return True

        for node in self.nodes:
            for disk in node.disks.list():
                # skip disks of wrong type
                if disk.type.name != self.disk_type:
                    continue
                # skip devices which have filesystems on the device
                if len(disk.filesystems) > 0:
                    continue

                # include devices which have partitions
                if len(disk.partitions) == 0:
                    available_disks.setdefault(node.name, []).append(disk)
                else:
                    if check_partition(disk):
                        # devices that have partitions with correct label will be in the beginning
                        available_disks.setdefault(node.name, []).insert(0, disk)
        return available_disks

    def start(self):
        logger.debug("start %s", self)
        for server in self.storage_servers:
            server.start()

    def stop(self):
        logger.debug("stop %s", self)
        for server in self.storage_servers:
            server.stop()

    def is_running(self):
        # TODO: Improve this, what about part of server running and part stopped
        for server in self.storage_servers:
            if not server.is_running():
                return False
        return True

    def health(self):
        """
        Return a view of the state all storage server running in this cluster
        example :
        {
        'cluster1_1': {'storageEngine': True, 'container': True},
        'cluster1_2': {'storageEngine': True, 'container': True},
        }
        """
        health = {}
        for server in self.storage_servers:
            running, _ = server.storageEngine.is_running()
            health[server.name] = {
                'storageEngine': running,
                'container': server.container.is_running(),
            }
        return health

    def __str__(self):
        return "StorageCluster <{}>".format(self.label)

    def __repr__(self):
        return str(self)


class StorageServer:
    """StorageEngine servers"""

    def __init__(self, cluster):
        self.cluster = cluster
        self.container = None
        self.storageEngine = None

    @classmethod
    def from_ays(cls, storageEngine_services, password=None):
        storageEngine = StorageEngine.from_ays(storageEngine_services, password)
        storage_server = cls(None)
        storage_server.container = storageEngine.container
        storage_server.storageEngine = storageEngine
        return storage_server

    @property
    def name(self):
        if self.storageEngine:
            return self.storageEngine.name
        return None

    @property
    def node(self):
        if self.container:
            return self.container.node
        return None

    def _find_port(self, start_port=2000):
        while True:
            if j.sal.nettools.tcpPortConnectionTest(self.node.addr, start_port, timeout=2):
                start_port += 1
                continue
            return start_port

    def start(self, timeout=30):
        logger.debug("start %s", self)
        if not self.container.is_running():
            self.container.start()

        ip, port = self.storageEngine.bind.split(":")
        self.storageEngine.bind = '{}:{}'.format(ip, self._find_port(port))
        self.storageEngine.start(timeout=timeout)

    def stop(self, timeout=30):
        logger.debug("stop %s", self)
        self.storageEngine.stop(timeout=timeout)
        self.container.stop()

    def is_running(self):
        container = self.container.is_running()
        storageEngine, _ = self.storageEngine.is_running()
        return (container and storageEngine)

    def __str__(self):
        return "StorageServer <{}>".format(self.container.name)

    def __repr__(self):
        return str(self)


class StorageDashboard:
    def __init__(self, cluster):
        self.cluster = cluster
        self.store = 'statsdb'

    def build_templating(self):
        templating = {
            "list": [],
            "rows": []
        }
        return templating

    def dashboard_template(self):
        return {
            "annotations": {
                "list": []
            },
            "editable": True,
            "gnetId": None,
            "graphTooltip": 0,
            "hideControls": False,
            "id": None,
            "links": [],
            "rows": [],
            "schemaVersion": 14,
            "style": "dark",
            "tags": [],
            "time": {
                "from": "now/d",
                "to": "now"
            },
            "timepicker": {
                "refresh_intervals": [
                    "5s",
                    "10s",
                    "30s",
                    "1m",
                    "5m",
                    "15m",
                    "30m",
                    "1h",
                    "2h",
                    "1d"
                ],
                "time_options": [
                    "5m",
                    "15m",
                    "1h",
                    "6h",
                    "12h",
                    "24h",
                    "2d",
                    "7d",
                    "30d"
                ]
            },
            "timezone": "",
            "title": self.cluster.name,
            "version": 8
        }

    def build_row(self, panel):
        template = {
            "collapse": False,
            "height": 295,
            "panels": [],
            "repeat": None,
            "repeatIteration": None,
            "repeatRowId": None,
            "showTitle": False,
            "title": "Dashboard Row",
            "titleSize": "h6"
        }
        template["panels"] += panel
        return template

    def build_panel(self, title, target, panel_id, unit):
        template = {
            "aliasColors": {},
            "bars": False,
            "dashLength": 10,
            "dashes": False,
            "datasource": self.store,
            "fill": 1,
            "id": panel_id,
            "legend": {
                "avg": False,
                "current": False,
                "max": False,
                "min": False,
                "show": True,
                "total": False,
                "values": False
            },
            "lines": True,
            "linewidth": 1,
            "links": [],
            "nullPointMode": "null",
            "percentage": False,
            "pointradius": 5,
            "points": False,
            "renderer": "flot",
            "seriesOverrides": [],
            "spaceLength": 10,
            "span": 6,
            "stack": True,
            "steppedLine": False,
            "targets": [],
            "thresholds": [],
            "timeFrom": None,
            "timeShift": None,
            "tooltip": {
                "shared": True,
                "sort": 0,
                "value_type": "individual"
            },
            "type": "graph",
            "xaxis": {
                "buckets": None,
                "mode": "time",
                "name": None,
                "show": True,
                "values": []
            },
            "yaxes": [
                {
                    "format": unit,
                    "label": None,
                    "logBase": 1,
                    "max": None,
                    "min": None,
                    "show": True
                },
                {
                    "format": "short",
                    "label": None,
                    "logBase": 1,
                    "max": None,
                    "min": None,
                    "show": True
                }
            ]
        }
        template["title"] = title
        template["targets"].append(target)
        return template

    def build_target(self, measurement, disks):
        template = {
            "alias": "$tag_node/$tag_id",
            "dsType": "influxdb",
            "groupBy": [
                {
                    "params": [
                        "$__interval"
                    ],
                    "type": "time"
                },
                {
                    "params": [
                        "node"
                    ],
                    "type": "tag"
                },
                {
                    "params": [
                        "id"
                    ],
                    "type": "tag"
                },
                {
                    "params": [
                        "none"
                    ],
                    "type": "fill"
                }
            ],
            "orderByTime": "ASC",
            "policy": "default",
            "rawQuery": False,
            "refId": "A",
            "resultFormat": "time_series",
            "select": [
                [
                    {
                        "params": [
                            "value"
                        ],
                        "type": "field"
                    },
                    {
                        "params": [],
                        "type": "mean"
                    }
                ]
            ],
            "tags": [
                {
                    "key": "type",
                    "operator": "=",
                    "value": "phys"
                }
            ]
        }
        template["measurement"] = measurement

        for idx, disk in enumerate(disks):
            tag = [
                {
                    "key": "node",
                    "operator": "=",
                    "value": disk.split("_")[0]
                },
                {
                    "condition": "AND",
                    "key": "id",
                    "operator": "=",
                    "value": disk.split("_")[1]
                }
            ]
            if idx == 0:
                tag[0]["condition"] = "AND"
            else:
                tag[0]["condition"] = "OR"
            template["tags"] += tag
        return template

    @property
    def template(self):
        AGGREGATED_CONFIG = {
            "Aggregated read IOPs": "disk.iops.read|m",
            "Aggregated write IOPs": "disk.iops.write|m",
            "Aggregated free size": "disk.size.free|m",
        }
        panel_id = 1
        disks = set()
        for server in self.cluster.storage_servers:
            server = server.name.split("_")
            disks.add("{}_{}".format(server[1], server[-3]))
        disks = list(disks)
        panels = []
        for title, measurement in AGGREGATED_CONFIG.items():
            if 'size' in title:
                partitions = [disk+'1' for disk in disks]
                target = self.build_target(measurement, partitions)
                panels.append(self.build_panel(title, target, panel_id, "decbytes"))
            else:
                target = self.build_target(measurement, disks)
                panels.append(self.build_panel(title, target, panel_id, "iops"))
            panel_id += 1

        for disk in disks:
            target = self.build_target("disk.iops.read|m", [disk])
            panels.append(self.build_panel("Read IOPs", target, panel_id, "iops"))
            panel_id += 1
            target = self.build_target("disk.iops.write|m", [disk])
            panels.append(self.build_panel("Write IOPs", target, panel_id, "iops"))
            panel_id += 1
            target = self.build_target("disk.size.free|m", [disk+'1'])
            panels.append(self.build_panel("Free size", target, panel_id, "decbytes"))
            panel_id += 1

        template = self.dashboard_template()
        for idx, panel in enumerate(panels):
            if idx % 2 == 0:
                row = self.build_row(panels[idx:idx+2])
                template["rows"].append(row)
        template = json.dumps(template)
        return template



import signal
import time
import requests
from js9 import j


class Grafana:
    def __init__(self, container, ip, port):
        self.container = container
        self.ip = ip
        self.port = port
        self.client = j.clients.grafana.get(url='http://%s:%d' % (
            ip, port), username='admin', password='admin')

    def apply_config(self):
        f = self.container.client.filesystem.open('/opt/grafana/conf/defaults.ini')
        try:
            template = self.container.client.filesystem.read(f)
        finally:
            self.container.client.filesystem.close(f)

        template = template.replace(b'3000', str(self.port).encode())
        self.container.client.filesystem.mkdir('/etc/grafana/')
        self.container.upload_content('/etc/grafana/grafana.ini', template)

    def is_running(self):
        for process in self.container.client.process.list():
            if 'grafana-server' in process['cmdline']:
                    return True, process['pid']
        return False, None

    def stop(self, timeout=30):
        is_running, pid = self.is_running()
        if not is_running:
            return

        self.container.client.process.kill(pid, signal.SIGTERM)
        start = time.time()
        end = start + timeout
        is_running, _ = self.is_running()
        while is_running and time.time() < end:
            time.sleep(1)
            is_running, _ = self.is_running()

        if is_running:
            raise RuntimeError('Failed to stop grafana.')

        if self.container.node.client.nft.rule_exists(self.port):
            self.container.node.client.nft.drop_port(self.port)

    def start(self, timeout=30):
        is_running, _ = self.is_running()
        if is_running:
            return

        self.apply_config()

        if not self.container.node.client.nft.rule_exists(self.port):
            self.container.node.client.nft.open_port(self.port)

        self.container.client.system(
            'grafana-server -config /etc/grafana/grafana.ini -homepath /opt/grafana')
        time.sleep(1)

        start = time.time()
        end = start + timeout
        is_running, _ = self.is_running()
        while not is_running and time.time() < end:
            time.sleep(1)
            is_running, _ = self.is_running()

        if not is_running:
            if self.container.node.client.nft.rule_exists(self.port):
                self.container.node.client.nft.drop_port(self.port)
            raise RuntimeError('Failed to start grafana.')

    def add_data_source(self, database, name, ip, port, count):
                data = {
                    'type': 'influxdb',
                    'access': 'proxy',
                    'database': database,
                    'name': name,
                    'url': 'http://%s:%u' % (ip, port),
                    'user': 'admin',
                    'password': 'passwd',
                    'default': True,
                }

                now = time.time()
                while time.time() - now < 10:
                    try:
                        self.client.addDataSource(data)
                        if len(self.client.listDataSources()) == count + 1:
                            continue
                        break
                    except requests.exceptions.ConnectionError:
                        time.sleep(1)
                        pass



import netaddr


def combine(ip1, ip2, mask):
    """
    >>> combine('10.0.3.11', '192.168.1.10', 24)
    '10.0.3.10'
    """
    iip1 = netaddr.IPNetwork('{}/{}'.format(ip1, mask))
    iip2 = netaddr.IPNetwork('{}/{}'.format(ip2, mask))
    ires = iip1.network + int(iip2.ip & (~ int(iip2.netmask)))
    net = netaddr.IPNetwork(ires)
    net.prefixlen = mask
    return net


class Network:
    def __init__(self, node):
        self.node = node
        self._client = node.client

    def get_management_info(self):
        import netaddr

        def get_nic_ip(nics, name):
            for nic in nics:
                if nic['name'] == name:
                    for ip in nic['addrs']:
                        return netaddr.IPNetwork(ip['addr'])
                    return

        defaultgwdev = self._client.bash("ip route | grep default | awk '{print $5}'").get().stdout.strip()
        nics = self._client.info.nic()
        mgmtaddr = None
        if defaultgwdev:
            ipgwdev = get_nic_ip(nics, defaultgwdev)
            if ipgwdev:
                mgmtaddr = str(ipgwdev.ip)
        if not mgmtaddr:
            mgmtaddr = self.addr

        return mgmtaddr

    def get_addresses(self, network):
        mgmtaddr = self.get_management_info()
        return {
            'storageaddr': combine(str(network.ip), mgmtaddr, network.prefixlen),
            'vxaddr': combine('10.240.0.0', mgmtaddr, network.prefixlen),
        }

    def get_free_nics(self):
        nics = self._client.info.nic()
        nics.sort(key=lambda nic: nic['speed'])
        availablenics = {}
        for nic in nics:
            # skip all interface that have an ipv4 address
            if any(netaddr.IPNetwork(addr['addr']).version == 4 for addr in nic['addrs'] if 'addr' in addr):
                continue
            if nic['speed'] == 0:
                continue
            availablenics.setdefault(nic['speed'], []).append(nic['name'])
        return sorted(availablenics.items())



import requests

class Oauth2ClientItsyouonline():
    def __init__(self, access_token_uri='https://itsyou.online/v1/oauth/access_token?response_type=id_token'):
        self.access_token_uri = access_token_uri

    def get_access_token(self, client_id, client_secret, scopes=[], audiences=[]):
        params = {
            'grant_type': 'client_credentials',
            'client_id': client_id,
            'client_secret': client_secret,
            'validity': 3600,
        }

        if len(scopes) > 0:
            params['scope'] = ",".join(scopes)
        if len(audiences) > 0:
            params['aud'] = ",".join(audiences)
        
        return requests.post(self.access_token_uri, params=params)


from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='0-orchestrator',
    version='1.1.0.a5',
    description='Zero-OS Orchestrator',
    long_description=long_description,
    url='https://github.com/g8os/grid',
    author='Christophe de Carvalho',
    author_email='christophe@gig.tech',
    license='Apache 2.0',
    packages=find_packages(),
    include_package_data=True,
    package_data={
        'zeroos.orchestrator.sal.templates': ['*', 'dashboards/*']
    },
    namespace_packages=['zeroos'],
    install_requires=['python-dateutil', 'Jinja2'],
)



"""
Auto-generated class for GWHost
"""
from .CloudInit import CloudInit

from . import client_support


class GWHost(object):
    """
    auto-generated. don't touch.
    """

    @staticmethod
    def create(hostname, ipaddress, macaddress, cloudinit=None, ip6address=None):
        """
        :type cloudinit: CloudInit
        :type hostname: str
        :type ip6address: str
        :type ipaddress: str
        :type macaddress: str
        :rtype: GWHost
        """

        return GWHost(
            cloudinit=cloudinit,
            hostname=hostname,
            ip6address=ip6address,
            ipaddress=ipaddress,
            macaddress=macaddress,
        )

    def __init__(self, json=None, **kwargs):
        if json is None and not kwargs:
            raise ValueError('No data or kwargs present')

        class_name = 'GWHost'
        create_error = '{cls}: unable to create {prop} from value: {val}: {err}'
        required_error = '{cls}: missing required property {prop}'

        data = json or kwargs

        property_name = 'cloudinit'
        val = data.get(property_name)
        if val is not None:
            datatypes = [CloudInit]
            try:
                self.cloudinit = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'hostname'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.hostname = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'ip6address'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.ip6address = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))

        property_name = 'ipaddress'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.ipaddress = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

        property_name = 'macaddress'
        val = data.get(property_name)
        if val is not None:
            datatypes = [str]
            try:
                self.macaddress = client_support.val_factory(val, datatypes)
            except ValueError as err:
                raise ValueError(create_error.format(cls=class_name, prop=property_name, val=val, err=err))
        else:
            raise ValueError(required_error.format(cls=class_name, prop=property_name))

    def __str__(self):
        return self.as_json(indent=4)

    def as_json(self, indent=0):
        return client_support.to_json(self, indent=indent)

    def as_dict(self):
        return client_support.to_dict(self)
